#! /usr/bin/env python

import pygame
import ode

IMAGE_FILE 	= 0
RECT_CENTER	= 1
BLOCK_SIZE = 16

class Sprite(pygame.sprite.Sprite):

	def __init__(self, centerPoint, image, image_file):
		pygame.sprite.Sprite.__init__(self)
		"""Set the image and the rect"""
		self.image = image
		self.rect = image.get_rect()
		"""Move the rect into the correct position"""
		self.rect.center = centerPoint
		#image filepath
		self.image_file = image_file
		#Dirty or not?
		self.dirty = True
		self.evManager = None


	def create_sprite_dict(self):
		"""This function is used to create a sprite dict
		that is used for saving sprites."""

		return	{
			IMAGE_FILE : self.image_file
			, RECT_CENTER : self.rect.center
		}

	def initialize_physics(self, world):
		"""Initialize your physical world"""
		self.world = world

	def update_position_from_rect(self):
		pass

class DiamondSprite(Sprite):

	def __init__(self, centerPoint, image, image_file):
		Sprite.__init__(self, centerPoint, image, image_file)

class PhysicsSprite(Sprite):

	def __init__(self, centerPoint, image, image_file):

		Sprite.__init__(self, centerPoint
									, image, image_file
									)
		self.world = None
		self.space = None
		self.body = None
		self.coord = None
		self.world_coord = None

	def initialize_physics(self, world, space, coord, world_coord):
		"""Initialize your physical world"""
		self.world = world
		self.space = space
		self.coord = coord
		self.world_coord = world_coord

	def set_event_manager(self, evManager):
		"""Set the event manager for the sprite.
		This lets you change it if want to
		"""
		#First un register
		if (self.evManager):
			self.evManager.UnregisterListener(self)
		#Now add to new manger
		self.evManager = evManager
		if (evManager):
			self.evManager.RegisterListener( self )

	def update_position_from_rect(self):

		if (self.world_coord):
			x,y = self.world_coord(*self.rect.center)
			if (self.body):
				self.body.setPosition((x, y,0))
			if (self.geom):
				self.geom.setPosition((x, y,0))


	def update(self):
		"""Update yourself"""

		if ((self.body) and (self.coord)):
			x_pos, y_pos, z = self.body.getPosition()
			self.body.setPosition((x_pos, y_pos, 0))
			self.rect.center = self.coord(x_pos, y_pos)

"""
************************************************
* Stone Block
************************************************
"""
class StoneBlock(PhysicsSprite):
	"""This represents a stone block"""

	def __init__(self, centerPoint, image, image_file):

		PhysicsSprite.__init__(self, centerPoint
									, image, image_file
									)


	def initialize_physics(self, world, space, coord, world_coord):
		"""Initialize your physical world"""

		PhysicsSprite.initialize_physics(self, world, space, coord, world_coord)

		self.create_geom()

	def create_geom(self):
		#Create the Stone Blocks Physics
		self.body = ode.Body(self.world)
		M = ode.Mass()
		b_x = 0.1
		b_y = 0.1
		b_z = 0.1
		M.setBox(750, b_x, b_y, b_z)
		self.body.setMass(M)


		#self.body.setGravityMode(False)
		x,y = self.world_coord(*self.rect.center)
		#print "x: %f y: %f c: %s" % (x, y, self.rect.center)
		self.body.setPosition((x, y,0))
		self.body.disable()

		self.geom = ode.GeomBox(self.space
			, lengths=(b_x, b_y, b_z))
		self.geom.movable = False
		self.geom.setBody(self.body)
		self.body.sprite = self

	def remove_geom(self):

		self.space.remove(geom)

	def update(self):
		"""Update yourself"""

		old_rect = pygame.Rect(self.rect)
		PhysicsSprite.update(self)
		if ((self.body) and (old_rect == self.rect)):
			self.body.disable()
