#! /usr/bin/env python

import pygame
import ode
import basicSprite
import swingEvent
from helpers import *
from constants import *

class Char(basicSprite.PhysicsSprite):
	"""This is our snake that will move around the screen"""

	def __init__(self, centerPoint, image, image_file):

		basicSprite.PhysicsSprite.__init__(self, centerPoint
									, image, image_file)

		"""Set the number of Pixels to move each time"""
		self.x_dist = 1
		self.y_dist = 1

		self.x_move = 0
		self.y_move = 0

		self.diamonds = 0

	def initialize_physics(self, world, space, coord, world_coord):
		"""Initialize your physical world"""

		basicSprite.PhysicsSprite.initialize_physics(
			self, world, space, coord, world_coord)

		#Create the character Physics
		self.body = ode.Body(world)
		M = ode.Mass()
		b_size, b_size2 = self.world_coord(BLOCK_SIZE, BLOCK_SIZE)
		b_x = 0.1
		b_y = 0.1
		b_z = 0.1
		M.setBox(1000, b_x, b_y, b_y)
		self.body.setMass(M)
		self.geom = ode.GeomBox(self.space
			, lengths=(b_x, b_y, b_y))
		if (self.world_coord):
			x,y = self.world_coord(*self.rect.center)
			self.body.setPosition((x, y,0))
			self.geom.setPosition((x, y,0))
			#self.body.disable()
		self.geom.setBody(self.body)
		self.geom.movable = True
		self.body.sprite = self
		#self.body.setGravityMode(False)

	def move(self, direction):
		"""Move yourself"""
		x,y,z = self.body.getForce()
		if (direction == DIRECTION_UP):
			pass
			#y = y + Y_FORCE
			#self.body.addForce((0,FORCE,0))
			#self.y_move += -self.y_dist
		elif (direction == DIRECTION_DOWN):
			self.y_move += self.y_dist
		elif (direction == DIRECTION_LEFT):
			x = x - X_FORCE
			#self.body.addForce((-FORCE,0,0))
			self.x_move += -self.x_dist
		elif (direction == DIRECTION_RIGHT):
			x = x + X_FORCE
			#self.body.addForce((FORCE,0,0))
			self.x_move += self.x_dist
		self.body.setForce((x,y,z))

	def update(self):
		"""Update yourself"""
		self.body.enable()
		old_rect = pygame.Rect(self.rect)
		basicSprite.PhysicsSprite.update(self)

		x,y = self.rect.center
		if (x > 640):
			x,y = self.world_coord(0,y)
			self.body.setPosition((x, y,0))
		elif (x < -BLOCK_SIZE):
			x,y = self.world_coord(640,y)
			self.body.setPosition((x, y,0))


		if (self.x_move>0):
			self.rect.move_ip(self.x_move, self.y_move)
			self.x_move = 0
			self.y_move = 0
			if (self.world_coord):
				x,y = self.world_coord(*self.rect.center)
				self.body.setPosition((x, y,0))
				self.geom.setPosition((x, y,0))

		if (old_rect.top != self.rect.top):
			new_event = swingEvent.CharactorMove(self, self.rect.center)
			self.evManager.Notify(new_event)

	def Notify(self, event):

		if isinstance(event, swingEvent.CharactorMoveRequest):
			self.move( event.direction )





