#! /usr/bin/env python

# swing up - a pyweek entry!
# Copyright (C) 2007 Mark Mruss <selsine@gmail.com>
# http://www.learningpython.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# If you find any bugs or have any suggestions email: selsine@gmail.com
# URL: http://pyweek.org/e/selsine_2_0/
import os, sys
import pygame
import data
import glob
import basicSprite
import charSprite
import cPickle
from pygame.locals import *
import spriteMap
from constants import *
if not pygame.font: print 'Warning, fonts disabled'
if not pygame.mixer: print 'Warning, sound disabled'


class LevelEdit(object):
	"""The Main PyMan Class - This class handles the main
	initialization and creating of the Game."""

	def __init__(self, level_file, screen_width=640, screen_height=480
				, level_width=640, level_height=640):
		"""Initialize"""
		"""Initialize PyGame"""
		pygame.init()
		"""Set the window Size"""
		self.screen_width = screen_width
		self.screen_height = screen_height
		"""Create the Screen"""
		self.screen = pygame.display.set_mode((self.screen_width
											, self.screen_height))

		self.tile_sprites = pygame.sprite.RenderUpdates()
		self.level_sprites = pygame.sprite.RenderUpdates()

		self.half_block = BLOCK_SIZE / 2

		tile_area_width = (BLOCK_SIZE* 4)
		x_pos = self.screen_width - tile_area_width
		self.tile_area = pygame.Rect(x_pos, 0, tile_area_width, self.screen_height)

		sprite_class_dict = {
			"char.png" : charSprite.Char
			, "rock.png" : basicSprite.Sprite
		}

		self.level_map = spriteMap.EditorMap(sprite_class_dict
				, 0, 0
				, (screen_width - self.tile_area.width)
				, screen_height
				, level_width, level_height)

		#Load the tile images
		self.load_tile_images()
		#Cursor stuff
		image, rect = data.load_image("cursor.png", (0,0,0))
		if (image):
			self.cursor = basicSprite.Sprite((self.half_block, self.half_block)
						, image, "cursor.png")
		self.last_cursor_rects = []
		#self.cursor_drag_rects = []
		self.last_drag_rect = None

		#selection sprite
		self.selected_tile = None
		self.level_map.load(level_file)
		self.level_file = level_file

	def load_tile_images(self):
		"""Load all of the tile images"""


		x_pos = self.tile_area.left + self.half_block
		y_pos = self.half_block
		for image_file in glob.glob(data.filepath(os.path.join("images","*.png"))):
			image, rect = data.load_image_full_path(image_file)
			if (image):
				head, tail = os.path.split(image_file)
				tile = basicSprite.Sprite((x_pos, y_pos), image, tail)
				self.tile_sprites.add(tile)
				x_pos = x_pos + BLOCK_SIZE
				if (x_pos > self.screen_width - BLOCK_SIZE):
					y_pos = y_pos + BLOCK_SIZE
					x_pos = self.tile_area.left + self.half_block
					#x_pos = self.screen_width - self.tile_area.left + self.half_block



	def MainLoop(self):
		"""This is the Main Loop of the Game"""

		"""Create the background"""
		self.background = pygame.Surface(self.screen.get_size())
		self.background = self.background.convert()
		self.background.fill((0,0,0))
		"""Draw the blocks onto the background, since they only need to be
		drawn once"""
		self.tile_sprites.draw(self.screen)
		self.tile_sprites.draw(self.background)
		#timer and font
		timer = pygame.time.Clock()
		font = None
		if pygame.font:
			font = pygame.font.Font(None, 36)
		pygame.display.flip()
		fps = 45
		while 1:
			for event in pygame.event.get():
				if event.type == pygame.QUIT:
					print timer.get_fps()
					sys.exit()
				elif event.type == KEYDOWN:
					if ((event.key == K_RIGHT)
					or (event.key == K_LEFT)
					or (event.key == K_UP)
					or (event.key == K_DOWN)):
						pass
				elif event.type == KEYUP:
					if (event.key == K_s):
						self.level_map.save(self.level_file)
				elif event.type == MOUSEMOTION:
					self.update_cursor_rect(event.pos)
					self.on_mouse_motion(event.pos, event.rel, event.buttons)
				elif event.type == MOUSEBUTTONDOWN:
					self.on_mouse_button_down(event.pos, event.button)
				elif event.type == MOUSEBUTTONUP:
					self.last_drag_rect = None

			self.screen.blit(self.background, (0, 0))

			reclist = [self.cursor.rect]

			#Last cursor rects (so we don't leave trasers
			#for rect in self.last_cursor_rects:
			#	reclist.append(rect)
			reclist.extend(self.last_cursor_rects)
			self.last_cursor_rects = []

			#Selected tile
			if (self.selected_tile):
				self.screen.blit(self.cursor.image, self.selected_tile.rect)
				reclist.append(self.selected_tile.rect)

			#Level Map

			reclist.extend(self.level_map.draw(self.screen))

			if pygame.font:
				font = pygame.font.Font(None, 36)
				#text = font.render("Pellets %s", 1, (255, 0, 0))
				#textpos = text.get_rect(centerx=self.background.get_width()/2)
				#self.screen.blit(text, textpos)
				s = "Frames Per Second: %.2f" % timer.get_fps()
				message = font.render(s, 1, (255,0,0))
				textpos = message.get_rect(centerx=self.background.get_width()/2)
				self.screen.blit(message, textpos)
				textpos.inflate_ip(20,0)
				reclist.append(textpos)

			#cursor
			self.screen.blit(self.cursor.image, self.cursor.rect)
			#print reclist
			pygame.display.update(reclist)
			#pygame.display.update([textpos])
			#pygame.display.flip()
			pygame.time.wait(5)
			#timer.tick()
			timer.tick(fps)

	def get_cursor_pos(self, pos):
		"""Get the cursor post, basically the BLOCK_SIZE x BLOCK_SIZE
		mapped rect position.
		"""
		if (self.tile_area.collidepoint(pos[0], pos[1])):
			return ((pos[0] - (pos[0] % BLOCK_SIZE))
				, (pos[1] - (pos[1] % BLOCK_SIZE)))
		else:
			return self.level_map.get_cursor_pos(pos)


	def update_cursor_rect(self, pos):
		"""Update the cursor sprite"""

		x_pos, y_pos = self.get_cursor_pos(pos)
		if (self.cursor.rect.topleft != (x_pos, y_pos)):
			self.last_cursor_rects.append(pygame.Rect(self.cursor.rect))
			self.cursor.rect.topleft = (x_pos, y_pos)

	def on_mouse_button_down(self, pos, button):

		x_pos, y_pos = self.get_cursor_pos(pos)
		if (button == 1):
			if (self.tile_area.collidepoint(x_pos, y_pos)):
				for sprite in self.tile_sprites:
					if (sprite.rect.collidepoint(pos[0], pos[1])):
						#WE hit the sprite
						if (self.selected_tile):
							#There was an oldone, redraw it
							self.last_cursor_rects.append(
								pygame.Rect(self.selected_tile.rect))
						self.selected_tile = sprite
			else:
				#in the level map
				if (pygame.key.get_mods() & KMOD_SHIFT):
					#Deleting!
					self.level_map.delete_sprite_at(x_pos, y_pos)
				else:
					#Adding
					self.add_selected_tile_to_map(x_pos, y_pos)

	def add_selected_tile_to_map(self, left, top):
		"""Helper function to add the selected tile to the map.
		it will update the drag rect.
		@param left - number - left side of rect where sprite
		will be positioned
		@param top - number = top coord of rect where sprite
		will be positioned
		"""
		if (self.selected_tile):
			self.add_sprite_to_map(self.selected_tile, left, top)
			self.last_drag_rect = pygame.Rect(self.cursor.rect)

	def add_sprite_to_map(self, sprite, left, top):
		"""Helper function to add the selected tile to the map.
		it will update the drag rect.
		@param left - number - left side of rect where sprite
		will be positioned
		@param top - number = top coord of rect where sprite
		will be positioned
		"""

		if (sprite):
			self.level_map.add_image_sprite(sprite
				, (left + self.half_block)
				, (top + self.half_block))

			#self.level_map.add_sprite(
					#basicSprite.Sprite(
						#(left + self.half_block, top + self.half_block)
						#, sprite.image))

	def on_mouse_motion(self, pos, rel, buttons):

		if (buttons[2]):
			self.level_map.move(*rel)
		elif ((buttons[0]) and (self.selected_tile)):
			#This is to allow "drag drawing"
			if (self.last_drag_rect != self.cursor.rect):
				#WE have moved so either add or delete drag
				x_pos, y_pos = self.get_cursor_pos(pos)
				if (pygame.key.get_mods() & KMOD_SHIFT):
					#Deleting!
					self.level_map.delete_sprite_at(x_pos, y_pos)
				else:
					#Adding
					self.add_selected_tile_to_map(x_pos, y_pos)



if __name__ == "__main__":
    editor = LevelEdit("level_01.swul")
    editor.MainLoop()
