#! /usr/bin/env python

# swing up - a pyweek entry!
# Copyright (C) 2007 Mark Mruss <selsine@gmail.com>
# http://www.learningpython.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# If you find any bugs or have any suggestions email: selsine@gmail.com
# URL: http://pyweek.org/e/selsine_2_0/

import os, sys
import pygame
import data
import spriteMap
import charSprite
import basicSprite
try:
	import ode
except:
	print "you need to install PyODE!"
	sys.exit()

import swingEvent
from helpers import *
from constants import *
from pygame.locals import *

if not pygame.font: print 'Warning, fonts disabled'
if not pygame.mixer: print 'Warning, sound disabled'


class SwingUp(object):
	"""This is the main game class"""

	def __init__(self, level_file, screen_width=640, screen_height=480):
		"""Initialize"""
		"""Initialize PyGame"""
		pygame.init()
		pygame.key.set_repeat(10,10)
		"""Set the window Size"""
		self.screen_width = screen_width
		self.screen_height = screen_height
		"""Create the Screen"""
		self.screen = pygame.display.set_mode((self.screen_width
											, self.screen_height))

		self.screen_width = screen_width
		self.screen_height = screen_height

		sprite_class_dict = {
			"char.png" : charSprite.Char
			, "rock.png" : basicSprite.StoneBlock
			, "diamond.png" : basicSprite.DiamondSprite
		}

		self.evManager = swingEvent.Manager()
		#self.evManager.RegisterListener(self)

		fps = 35
		self.physics = SwingPhysics(fps)
		self.level_map = spriteMap.GameMap(self.evManager
				, sprite_class_dict
				, fps
				, 0, 0
				, screen_width
				, screen_height
				, screen_width, screen_height)

		self.level_map.load(level_file)
		self.level_file = level_file

		y_pos = self.screen_height - self.level_map.rect.height
		self.level_map.move(0, y_pos)

		self.level_map.initialize_physics(
			self.physics.world
			, self.physics.space
			, self.physics.coord
			, self.physics.world_coord)

	def run(self):
		"""This is the Main Loop of the Game"""

		self.screen.blit(self.level_map.background
			, (0,0))
		pygame.display.flip()
		while 1:

			self.event_handler()

			self.level_map.draw(self.screen)

			self.physics.simulate()

	def event_handler(self):

		for event in pygame.event.get():
			new_event = None
			if event.type == pygame.QUIT:
				sys.exit()
			elif event.type == KEYDOWN \
				and event.key == K_ESCAPE:
				sys.exit()
			elif event.type == KEYDOWN:
				if event.key == K_w:
					new_event = swingEvent.CharactorMoveRequest(DIRECTION_UP)
				elif event.key == K_s:
					new_event = swingEvent.CharactorMoveRequest(DIRECTION_DOWN)
				elif event.key == K_a:
					new_event = swingEvent.CharactorMoveRequest(DIRECTION_LEFT)
				elif event.key == K_d:
					new_event = swingEvent.CharactorMoveRequest(DIRECTION_RIGHT)
			elif event.type == MOUSEMOTION:
				pass#new_event = swingEvent.MouseMove(event.pos)
			elif event.type == MOUSEBUTTONDOWN:
				new_event = swingEvent.MouseButtonDown(event.pos, event.button)
			elif event.type == MOUSEBUTTONUP:
				new_event = swingEvent.MouseButtonUp(event.pos, event.button)
			if new_event:
				self.evManager.Notify(new_event)

class SwingPhysics(object):
	"""THis class is responsible for the physics of the game.
	I'm not sure if it should be here or not.
	"""

	def __init__(self, fps):

		self.world = None
		self.fps = fps
		self.dt = 1.0/self.fps
		self.contactgroup = None

		self.initialize_world()

	def initialize_world(self):
		"""Initialize the physics
		"""
		# Create a world object
		self.world = ode.World()
		#self.world.setQuickStepNumIterations(20)
		self.world.setAutoDisableFlag(True)
		self.world.setContactSurfaceLayer(0.001)
		self.world.setGravity((0,-9.81,0))
		#COntact group for collisions
		self.contactgroup = ode.JointGroup()
		# Create a space object
		self.space = ode.Space(type=1)

		floor = ode.GeomPlane(self.space,normal=(0,1,0),dist=-0.40)
		#ceiling = ode.GeomPlane(self.space,normal=(0,6,0),dist=1)
		#frontwall = ode.GeomPlane(self.space,normal=(0,0,1),dist=-1)
		#backwall = ode.GeomPlane(self.space,normal=(0,0,-1),dist=-1)
		#left_side = ode.GeomPlane(self.space,normal=(1,0,0),dist=-1.8)
		#right_side = ode.GeomPlane(self.space,normal=(1,0,0),dist=-9)
		#ceiling.movable = False
		floor.movable = False
		#frontwall.movable = False
		#backwall.movable = False
		##left_side.movable = False
		#right_side.movable = False

	def simulate(self):
		# Next simulation step
		#self.world.step(self.dt)

		# Simulate
		n = 2
		for i in range(n):
			# Detect collisions and create contact joints
			self.space.collide((self.world, self.contactgroup)
				, self.near_callback)

			# Simulation step
			self.world.quickStep(self.dt/n)
			#self.world.step(self.dt/n)
			# Remove all contact joints
			self.contactgroup.empty()



	# Collision callback
	def near_callback(self, args, geom1, geom2):
		"""Callback function for the collide() method.

		This function checks if the given geoms do collide and
		creates contact joints if they do.
		"""

		body1 = None
		body2 = None
		if ((not geom1.movable) and
			(not geom2.movable)):
			return
		else:
			if (geom1.movable):
				body1 = geom1.getBody()
			if (geom2.movable):
				body2 = geom2.getBody()
			if (not body1) and (not body2):
				return
		# Check if the objects do collide
		contacts = ode.collide(geom1, geom2)
		# Create contact joints
		world,contactgroup = args
		for c in contacts:
			#print "Sprite: %s Sprite: %s" % (
			#	geom1.getBody().sprite, geom2.getBody().sprite)
			c.setBounce(0.4)
			c.setMu(50)
			j = ode.ContactJoint(world, contactgroup, c)
			j.attach(body1, body2)

	def coord(self, x,y):
		"""Convert world coordinates to pixel coordinates."""
		return 320+170*x, 400-170*y

	def world_coord(self, x, y):
		"""pixel to world"""
		return ((x - 320.0)/170.0), ((y - 400)/-170.0)

def main():
   swing_up = SwingUp("level_01.swul")
   swing_up.run()

if __name__ == "__main__":
    main()
