#! /usr/bin/env python

# swing up - a pyweek entry!
# Copyright (C) 2007 Mark Mruss <selsine@gmail.com>
# http://www.learningpython.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#
# If you find any bugs or have any suggestions email: selsine@gmail.com
# URL: http://pyweek.org/e/selsine_2_0/
import os, sys
import pygame
import data
import glob
import basicSprite
import cPickle
import charSprite
import ode
import swingEvent
from constants import *

if not pygame.font: print 'Warning, fonts disabled'
if not pygame.mixer: print 'Warning, sound disabled'

class BaseMap(object):

	def __init__(self, sprite_class_dict, map_x, map_y
				, screen_width=640, screen_height=480
				, map_width=640, map_height=640):
		"""Initialize the map.
		@param sprite_class_dict - dictionary - a confusing dictionary
		of image filenames to sprite classes.  So when we create a
		sprite based on an image, we use the correct sprite class.
		@param map_x - number the x position of the maps top left
		@param map_y - number the y position of the maps top right
		@param screen_height - number the height of the pygame
		window
		@param screen_width - number the width of the pygame
		window
		@param map_width - number - the width of the map
		@param map_height - number - the height of the map.
		"""

		#position information
		self.map_x = map_x
		self.map_y = map_y
		self.screen_rect = pygame.Rect(0, 0, screen_width, screen_height)

		self.sprite_class_dict = sprite_class_dict

		#initialize the values
		self.initialize_values(map_width, map_height)

	def initialize_values(self, map_width, map_height):

		#sprite list
		self.sprites = pygame.sprite.OrderedUpdates()
		#Map rect
		self.rect = pygame.Rect(self.map_x, self.map_y, map_width, map_height)

		self.image = pygame.Surface((map_width, map_height))
		self.image = self.image.convert()
		self.image.fill((170,170,170))
		#Offsets for scrolling
		self.x_pos = 0
		self.y_pos = 0

		#Rects to redraw
		self.draw_rects = [self.rect]


	"""
	*************************************************
	* Saves and Loads
	****************************************************
	"""
	def load(self, filename):
		"""This function loads a level from a file
		@param filename - string - the name ofthe level file to
		load
		@returns boolean - success or failure"""
		#reset the variables

		#try:
		level_file = data.load(filename, "rb")
		map_width = cPickle.load(level_file)
		map_height = cPickle.load(level_file)
		self.initialize_values(map_width, map_height)

		#Load the sprite list
		sprite_list = cPickle.load(level_file)

		level_file.close()
		return self.load_from_sprite_list(sprite_list)

		#except cPickle.UnpicklingError, e:
		#	print ("Error opening file: %s\r\n%s" % (filename, e))
		#	return False
		#except:
		#	print ("Error opening file: %s" % filename)
		#	return False

	def save(self, filename):
		"""Save the current level to filename
		@param filename - string - the file name to save
		the level too.
		@returns boolean - success or failure
		"""
		try:
			level_file = data.load(filename, "w")
			cPickle.dump(self.rect.width, level_file, 0)
			cPickle.dump(self.rect.height, level_file, 0)
			cPickle.dump(self.save_to_sprite_list(), level_file, 0)
			level_file.close()
			return True
		except cPickle.PicklingError, e:
			print ("Error saving file: %s\r\n%s") % (filename, e)
			return False
		except:
			print ("Error saving file: %s" % filename)
			return False

	def load_from_sprite_list(self, sprite_list):
		"""Load all of the sprites from a sprite list.
		@param sprite_list - list - A list of sprite dictionaries
		that give us the information needed to load our sprites.
		@returns boolean - success or failure.
		"""
		"""
		IMAGE_FILE 	= 0		RECT_LEFT	= 1	RECT_TOP	= 2
		"""

		image_dict = {}

		for sprite_dict in sprite_list:
			image_file = sprite_dict[basicSprite.IMAGE_FILE]
			if (not image_dict.has_key(image_file)):
				image_dict[image_file], rect = data.load_image(image_file)
			new_sprite = self.create_sprite(sprite_dict[basicSprite.RECT_CENTER]
					, image_dict[image_file]
					, image_file)
			self.add_sprite_to_lists(new_sprite)

	def save_to_sprite_list(self):
		"""This function saves all the sprite date into
		a list of sprit dicts.
		@returns - list - a list of sprite dicts
		"""

		sprite_list = []

		for sprite in self.sprites:
			dict = sprite.create_sprite_dict()
			sprite_list.append(dict)
		return sprite_list

	"""
	****************************************************
	* Sprite Functions
	****************************************************
	"""

	def add_sprite_to_lists(self, sprite):
		"""private function to add sprites to
		the lists.  Added so that sub classed can override
		the way that things happen
		"""
		self.sprites.add(sprite)
		self.draw_sprite(sprite)

	def create_sprite(self, centerPoint, image, image_file):
		"""Helper function to create the sprite so that we
		don't have to duplicate dictonary lookups.
		"""
		#is it in the dict?
		if (self.sprite_class_dict.has_key(image_file)):
			new_sprite = self.sprite_class_dict[image_file](
				centerPoint, image, image_file)
		else:
			new_sprite = basicSprite.Sprite(
				centerPoint, image, image_file)

		return new_sprite

	def add_image_sprite(self, old_sprite, x_screen, y_screen):

		#Mape the coords
		x_pos, y_pos = self.screen_to_map_point(x_screen, y_screen)
		#creat the sprite
		new_sprite = self.create_sprite((x_pos, y_pos)
					, old_sprite.image, old_sprite.image_file)
		self.add_sprite_to_lists(new_sprite)

	def add_sprite(self, sprite):
		"""Add a sprite to the map"""

		self.add_sprite_to_lists(sprite)

	def delete_sprite_at(self, x_pos, y_pos):
		"""delete all sprites
		"""
		x_pos, y_pos = self.screen_to_map_point(x_pos, y_pos)
		for sprite in reversed(self.sprites.sprites()):
			if (sprite.rect.collidepoint(x_pos, y_pos)):
				self.sprites.remove(sprite)
				self.update_image()
				#done!
				break


	def update_image(self):
		"""This function will redraw the image.  Only really
		called when a map is loaded or an item is removed from the
		map"""
		self.image = pygame.Surface((self.rect.width, self.rect.height))
		self.image = self.image.convert()
		self.image.fill((170,170,170))
		for sprite in self.sprites:
			self.draw_sprite(sprite)

	"""
	******************************************************
	* Movement and Coord functions
	******************************************************
	"""
	def move(self, x_amount, y_amount):

		self.x_pos = self.x_pos + x_amount
		self.y_pos = self.y_pos + y_amount

	def map_to_screen_point(self, x_pos, y_pos):
		"""Convert screen coords to map coords"""

		return ((x_pos - self.rect.left - self.x_pos)
				, (y_pos - self.rect.top - self.y_pos))

	def screen_to_map_point(self, x_pos, y_pos):
		"""Convert screen coords to map coords"""

		return ((x_pos - self.rect.left - self.x_pos)
			, (y_pos - self.rect.top - self.y_pos))

	def draw_sprite(self, sprite):
		"""render the sprite"""
		self.image.blit(sprite.image, sprite.rect.topleft)

	def draw(self, surface):

		rect = pygame.Rect(0,0, (self.screen_rect.right - self.x_pos)
			, (self.screen_rect.bottom - self.y_pos))
		surface.blit(self.image, (self.x_pos, self.y_pos), rect)

		list_copy = []
		for rect in self.draw_rects:
			list_copy.append(pygame.Rect(rect))
		self.draw_rects = []
		return list_copy

	def get_cursor_pos(self, pos):
		"""Get the cursor post, basically the BLOCK_SIZE x BLOCK_SIZE
		mapped rect position.
		"""
		#Get the x and y positions regardles of the offset
		x_return = (pos[0] - (pos[0] % BLOCK_SIZE))
		y_return = (pos[1] - (pos[1] % BLOCK_SIZE))
		#get the offset modul
		x_mod = (self.x_pos % BLOCK_SIZE)
		y_mod = (self.y_pos % BLOCK_SIZE)
		"""now ensure that we move in the correct direction
		the results cannot be bigger then the passed in coords"""
		if ((x_return + x_mod) > pos[0]):
			x_return = x_return - (BLOCK_SIZE - x_mod)
		else:
			x_return = x_return + x_mod

		if ((y_return + y_mod) > pos[1]):
			y_return = y_return - (BLOCK_SIZE - y_mod)
		else:
			y_return = y_return + y_mod

		return (x_return, y_return)

	def get_sprite(self, sprite_type, remove_from_list = False):
		"""This function returns the FIRST sprite
		in the list that matches this passed sprite type.
		If remove_from_list == True then the sprite will be
		removed from the list and returned.
		"""
		sprite_return = None
		for sprite in self.sprites:
			if (isinstance(sprite, sprite_type)):
				sprite_return = sprite
				if (remove_from_list):
					self.sprites.remove(sprite)
					self.update_image()
				break

		return sprite_return
"""
***********************************************************
* Editor Map
***********************************************************
"""
class EditorMap(BaseMap):
	"""This is a map to be used by the Level editor.
	it doesn't care about moving sprites or anything like
	that.
	"""

	def __init__(self, sprite_class_dict, map_x, map_y
				, screen_width=640, screen_height=480
				, map_width=640, map_height=640):

		"""Initialize the map.
		@param sprite_class_dict - dictionary - a confusing dictionary
		of image filenames to sprite classes.  So when we create a
		sprite based on an image, we use the correct sprite class.
		@param map_x - number the x position of the maps top left
		@param map_y - number the y position of the maps top right
		@param screen_height - number the height of the pygame
		window
		@param screen_width - number the width of the pygame
		window
		@param map_width - number - the width of the map
		@param map_height - number - the height of the map.
		"""

		BaseMap.__init__(self, sprite_class_dict, map_x, map_y
				, screen_width, screen_height
				, map_width, map_height)

		#Rects to redraw
		self.draw_rects.append(self.rect)

	"""
	****************************************************
	* Sprite Functions
	****************************************************
	"""
	def create_sprite(self, centerPoint, image, image_file):
		"""Helper function to create the sprite so that we
		don't have to duplicate dictonary lookups.
		"""
		#is it in the dict?
		if (self.sprite_class_dict.has_key(image_file)):
			new_sprite = self.sprite_class_dict[image_file](
				centerPoint, image, image_file)
		else:
			new_sprite = basicSprite.Sprite(
				centerPoint, image, image_file)

		return new_sprite


	def update_image(self):
		"""This function will redraw the image.  Only really
		called when a map is loaded or an item is removed from the
		map"""
		BaseMap.update_image(self)
		self.draw_rects = [self.rect]

	"""
	******************************************************
	* Movement and Coord functions
	******************************************************
	"""
	def move(self, x_amount, y_amount):
		BaseMap.move(self, x_amount, y_amount)
		self.draw_rects = [self.rect]
"""
**************************************************************
* GameMap
**************************************************************
"""
class GameMap(BaseMap):
	"""
	This is an interactive map designed to be used for game play
	"""

	#Hand Cursor
	__hand_cursor_string = (
		"     XX         ",
		"    X..X        ",
		"    X..X        ",
		"    X..X        ",
		"    X..XXXXX    ",
		"    X..X..X.XX  ",
		" XX X..X..X.X.X ",
		"X..XX.........X ",
		"X...X.........X ",
		" X.....X.X.X..X ",
		"  X....X.X.X..X ",
		"  X....X.X.X.X  ",
		"   X...X.X.X.X  ",
		"    X.......X   ",
		"     X....X.X   ",
		"     XXXXX XX   ")
	__hcurs, __hmask = pygame.cursors.compile(__hand_cursor_string, ".", "X")
	__hand = ((16, 16), (5, 1), __hcurs, __hmask)

	def __init__(self,evManager, sprite_class_dict, fps, map_x, map_y
				, screen_width=640, screen_height=480
				, map_width=640, map_height=640):
		"""Initialize the map.
		@param sprite_class_dict - dictionary - a confusing dictionary
		of image filenames to sprite classes.  So when we create a
		sprite based on an image, we use the correct sprite class.
		@param map_x - number the x position of the maps top left
		@param map_y - number the y position of the maps top right
		@param screen_height - number the height of the pygame
		window
		@param screen_width - number the width of the pygame
		window
		@param map_width - number - the width of the map
		@param map_height - number - the height of the map.
		"""

		BaseMap.__init__(self, sprite_class_dict, map_x, map_y
				, screen_width, screen_height
				, map_width, map_height)



		self.evManager = evManager
		self.evManager = evManager
		if (evManager):
			self.evManager.RegisterListener( self )
		self.timer = pygame.time.Clock()
		#self.game_surface = None

		#character sprite
		self.char_sprite = None
		#Moving Game Sprites
		self.game_sprites = pygame.sprite.RenderUpdates()
		#block sprites
		self.block_sprites = pygame.sprite.RenderUpdates()
		#jewel sprites
		self.jewel_sprites = pygame.sprite.RenderUpdates()

		self.text_rects = []

		self.sprites = pygame.sprite.RenderUpdates()

		self.fps = fps

		#Physics
		self.world = None
		self.space = None
		self.body = None
		self.coord = None
		self.world_coord = None
		self.physics_init = False
		self.ball_joint = None
		self.joint_group = ode.JointGroup()
		self.grapple_joint = None
		self.highlight = False
		self.grapple_rect = None

	def initialize_values(self, map_width, map_height):
		"""Called when the map needs to initialize itself.
		WE will call the base class and then create our
		background surface
		"""

		BaseMap.initialize_values(self, map_width, map_height)

		self.background = pygame.Surface((map_width, map_height))
		self.background = self.image.convert()
		self.background.fill((170,170,170))
		rect = pygame.Rect(0,0, (self.screen_rect.right - self.x_pos)
					, (self.screen_rect.bottom - self.y_pos))
		self.background.blit(self.image, (self.x_pos, self.y_pos), rect)

		self.interior_rect = self.rect.inflate(
			-(BLOCK_SIZE * 2), -(BLOCK_SIZE * 2))

	"""
	****************************************************
	* Sprite Functions
	****************************************************
	"""

	def add_sprite_to_lists(self, sprite):
		"""private function to add sprites to
		the lists.  Added so that sub classed can override
		the way that things happen
		"""

		if (isinstance(sprite, charSprite.Char)):
			self.game_sprites.add(sprite)
			sprite.set_event_manager(self.evManager)
			self.char_sprite = sprite
		elif (isinstance(sprite, basicSprite.StoneBlock)):
			if (sprite.rect.colliderect(self.interior_rect)):
				self.game_sprites.add(sprite)
			else:
				self.block_sprites.add(sprite)
		elif (isinstance(sprite, basicSprite.DiamondSprite)):
			self.jewel_sprites.add(sprite)
		else:
			self.sprites.add(sprite)
			self.draw_sprite(sprite)

	def update_image(self):
		"""This function will redraw the image.  Only really
		called when a map is loaded or an item is removed from the
		map"""
		BaseMap.update_image(self)
		self.draw_rects = [self.rect]
		rect = pygame.Rect(0,0, (self.screen_rect.right - self.x_pos)
					, (self.screen_rect.bottom - self.y_pos))
		self.background.blit(self.image, (self.x_pos, self.y_pos), rect)

	"""
	******************************************************
	* Movement and Coord functions
	******************************************************
	"""
	def move(self, x_amount, y_amount):
		BaseMap.move(self, x_amount, y_amount)

		for sprite in self.game_sprites:
			sprite.rect.move_ip(x_amount, y_amount)
			sprite.update_position_from_rect()

		for sprite in self.sprites:
			sprite.rect.move_ip(x_amount, y_amount)
			sprite.update_position_from_rect()

		for sprite in self.block_sprites:
			sprite.rect.move_ip(x_amount, y_amount)
			sprite.update_position_from_rect()

		for sprite in self.jewel_sprites:
			sprite.rect.move_ip(x_amount, y_amount)
			sprite.update_position_from_rect()


		self.draw_rects = [self.rect]

	"""
	******************************************************
	* Drawing
	******************************************************
	"""
	def draw(self, surface):
		"print drawing!"
		dirty_rects = []
		#Erase the last text rect
		for rect in self.text_rects:
			surface.blit(self.background, rect, rect)
			dirty_rects.append(rect)
		self.text_rects = []

		if (self.grapple_rect):
			surface.blit(self.background
				, self.grapple_rect, self.grapple_rect)
			dirty_rects.append(self.grapple_rect)
			self.grapple_rect = None
		self.block_sprites.clear(surface, self.background )
		self.game_sprites.clear(surface, self.background )
		self.jewel_sprites.clear(surface, self.background)

		diamonds = pygame.sprite.spritecollide(self.char_sprite
			, self.jewel_sprites, True)

		self.char_sprite.diamonds += len(diamonds)

		self.block_sprites.update()
		self.game_sprites.update()
		self.jewel_sprites.update()
		#Draw the grapples
		self.draw_grapple(surface)

		dirty_rects = (self.block_sprites.draw(surface)
			+ self.game_sprites.draw(surface)
			+ self.jewel_sprites.draw(surface))
		grapple_rect = self.draw_grapple(surface)
		if (grapple_rect):
			self.grapple_rect = grapple_rect
			dirty_rects.append(self.grapple_rect)


		if pygame.font:
			font = pygame.font.Font(None, 36)

			s = "Frames Per Second: %.2f" % self.timer.get_fps()
			message = font.render(s, 1, (255,0,0))
			textpos = message.get_rect(left=10)
			surface.blit(message, textpos)
			self.text_rects.append(textpos)
			textpos.inflate_ip(20,0)
			dirty_rects.append(textpos)

			s = "Diamonds: %d" % self.char_sprite.diamonds
			message = font.render(s, 1, (255,0,0))
			textpos = message.get_rect(right=surface.get_width() - 10 )
			surface.blit(message, textpos)
			self.text_rects.append(textpos)
			textpos.inflate_ip(20,20)
			dirty_rects.append(textpos)

		pygame.display.update( dirty_rects )
		pygame.time.wait(5)
		self.timer.tick(self.fps)

		return dirty_rects

	def draw_grapple(self, surface):

		rect_return = None
		if ((self.grapple_joint)
			and (self.ball_joint)
			and (self.char_sprite)):
			x, y, z = self.ball_joint.getAnchor()
			rect_return = pygame.draw.aaline(surface
				, (55,0,200)
				, self.char_sprite.rect.center
				, self.coord(x,y), 5)

		return rect_return


	def initialize_physics(self, world, space, coord, world_coord):

		self.physics_init = True
		self.world = world
		self.space = space
		self.coord = coord
		self.world_coord = world_coord

		for sprite in self.game_sprites:
			sprite.initialize_physics(world, space, coord, world_coord)

	""""
	****************************************************
	" Event Handlers
	****************************************************
	"""

	def Notify(self, event):

		if isinstance(event, swingEvent.MouseMove):
			self.set_cursor(event.pos)
		elif (isinstance(event, swingEvent.MouseButtonDown)):
			self.on_mouse_button_down(event.pos, event.button)
		elif (isinstance(event, swingEvent.MouseButtonUp)):
			self.on_mouse_button_up(event.pos, event.button)
		elif (isinstance(event, swingEvent.CharactorMove)):
			x,y = event.pos
			#print self.y_pos, y
			#y = (self.y_pos - y)
			#print y
			#if (y > -300):
				#self.move(0,10)
			#elif (y < -500):
				#self.move(0,10)
			#elif (y < -self.rect.height):
				#self.move(0,(y + self.rect.height))



	def on_mouse_button_down(self, pos, button):
		"""Check to see if we need the cursor"""

		connect_sprite = None
		if ((button == LEFT_BUTTON)
			and (self.char_sprite)
			and (self.physics_init)):
			for sprite in self.game_sprites:
				if (( not isinstance(sprite, charSprite.Char))
					and (sprite.rect.collidepoint(*pos))):
					connect_sprite = sprite
					break
			if (connect_sprite):
				"""First create the a body to attach a ball joing to.
				This will give the rope it's "swing".
				"""
				x, y = self.world_coord(*connect_sprite.rect.center)
				body = ode.Body(self.world)
				M = ode.Mass()
				b_x = 0.1
				b_y = 0.1
				b_z = 0.1
				M.setBox(1000, b_x, b_y, b_y)
				body.setMass(M)
				body.setPosition((x, y,0))

				self.ball_joint = ode.BallJoint(self.world
					, self.joint_group)
				self.ball_joint.attach(body
					 , ode.environment)
				self.ball_joint.setAnchor((x , y, 0))

				"""Then create the sliderJoint wich will create the
				grapple effect.
				"""
				x1, y1 = self.world_coord(*connect_sprite.rect.center)
				x2, y2 = self.world_coord(*self.char_sprite.rect.center)
				self.grapple_joint = ode.SliderJoint(self.world
					, self.joint_group)
				self.grapple_joint.attach(self.char_sprite.body
					, body)
				#self.grapple_joint.setAnchor((x , y, 0))
				self.grapple_joint.setAxis( (x1-x2, y1-y2, 0) )
				self.grapple_joint.setParam(ode.ParamVel, 2)
				self.grapple_joint.setParam(ode.ParamFMax, 15)


	def on_mouse_button_up(self, pos, button):
		"""Check to see if we need the cursor"""

		if ((self.grapple_joint)
			and (self.ball_joint)
			and (self.physics_init)):
				body = self.ball_joint.getBody(0)
				del body
				self.joint_group.empty()
				self.grapple_joint = None


	def set_cursor(self, pos):
		"""Check to see if we need the cursor"""
		self.highlight = False
		for sprite in self.game_sprites:
			if (( not isinstance(sprite, charSprite.Char))
				and (sprite.rect.collidepoint(*pos))):
				self.highlight = True
				break
		if (self.highlight):
			pygame.mouse.set_cursor(*self.__hand)
		else:
			pygame.mouse.set_cursor(*pygame.cursors.arrow)





