
def Debug( msg ):
        pass#print msg

class Event(object):
	"""this is a superclass for any events that might be generated by an
	object and sent to the EventManager"""
	def __init__(self):
		self.name = "Generic Event"

class Tick(Event):
	def __init__(self):
		self.name = "CPU Tick Event"

class Quit(Event):
	def __init__(self):
		self.name = "Program Quit Event"

class MapBuilt(Event):
	def __init__(self, map):
		self.name = "Map Finished Building Event"
		self.map = map

class GameStarted(Event):
	def __init__(self, game):
		self.name = "Game Started Event"
		self.game = game

class GameSurfaceReady(Event):
	def __init__(self, surface):
		self.name = "Game Surface Read Event"
		self.surface = surface

class CharactorMoveRequest(Event):
	def __init__(self, direction):
		self.name = "Charactor Move Request"
		self.direction = direction

class PhysicsWorldCreated(Event):
	def __init__(self, world, space, coord, world_coord):
		self.name = "Physics World Created"
		self.world = world
		self.space = space
		self.coord = coord
		self.world_coord = world_coord

class PhysicsWorldReady(Event):
	def __init__(self, world, space, coord, world_coord):
		self.name = "Physics World Created"
		self.world = world
		self.space = space
		self.coord = coord
		self.world_coord = world_coord

class SetFPS(Event):
	def __init__(self, fps):
		self.name = "Set FPS"
		self.fps = fps

class CharactorPlace(Event):
	"""this event occurs when a Charactor is *placed* in a sector,
	ie it doesn't move there from an adjacent sector."""
	def __init__(self, charactor):
		self.name = "Charactor Placement Event"
		self.charactor = charactor

class CharactorMove(Event):
	def __init__(self, charactor, pos):
		self.name = "Charactor Move Event"
		self.charactor = charactor
		self.pos = pos

class MouseMove(Event):
	def __init__(self, pos):
		self.name = "Mouse move event"
		self.pos = pos

class MouseButtonDown(Event):
	def __init__(self, pos, button):
		self.name = "Mouse move event"
		self.pos = pos
		self.button = button

class MouseButtonUp(Event):
	def __init__(self, pos, button):
		self.name = "Mouse move event"
		self.pos = pos
		self.button = button

class Manager(object):
	"""this object is responsible for coordinating most communication
	between the Model, View, and Controller."""
	def __init__(self ):
		from weakref import WeakKeyDictionary
		self.listeners = WeakKeyDictionary()
		self.eventQueue= []

	#----------------------------------------------------------------------
	def RegisterListener( self, listener ):
		self.listeners[ listener ] = 1

	#----------------------------------------------------------------------
	def UnregisterListener( self, listener ):
		if listener in self.listeners.keys():
			del self.listeners[ listener ]

	#----------------------------------------------------------------------
	def Notify( self, ev):
		#if not isinstance(ev, Tick): Debug( "     Message: " + ev.name )
		for listener in self.listeners.keys():
			#If the weakref has died, remove it and continue
			#through the list
			if listener is None:
				del self.listeners[ listener ]
				continue
			listener.Notify( ev)
